/****************************************************************************************************/
/** @file
*
* @par Module:
*      bootloader.h
*
* @par Responsible:
*      Christian Gaertner
*
* @par Module Description:
*      header of the drv_can.c file
*
*****************************************************************************************************/
/*
* $Date: 2007-06-17 10:00:08 +0200 (So, 17 Jun 2007) $
* $Revision: 14 $
* $Author: Christian Gärtner $
* $HeadURL: file:///D:/SVN/Elektronikprojekte/trunk/Drivers/Includes/drv_can.h $
*
*****************************************************************************************************/

/*****************************************************************************************************
* Include files
*****************************************************************************************************/

/*****************************************************************************************************
* Module definition against multiple inclusion
*****************************************************************************************************/
#ifndef MODULE_DRV_CAN_H
#define MODULE_DRV_CAN_H

#ifdef MODULE_DRV_CAN_C
    #define MOD_EXTERN
#else
    #define MOD_EXTERN extern
#endif

/*****************************************************************************************************
* Definition of project wide MACROS / #DEFINE-CONSTANTS / INLINE-Functions
*****************************************************************************************************/
#define CAN_MAX_MSGBUFFER                   (3)

#define MESSAGE_BUFFER_LOCKED               (0x01u)         /* buffer occupied by message */
#define MESSAGE_RECEIVED_NEW                (0x02u)         /* message received, not processed yet */
#define MESSAGE_CYCLIC_SEND                 (0x40u)         /* send cyclic regarding the timer values */
#define MESSAGE_READY_TO_SEND               (0x80u)         /* send only a single time */

/**
 *  @note CAN_MARK_EXTENDED_ID - internal definition for extended IDs.
 *        The MSB is not used for the identifier, only for internal
 *        indicaton and handling.
 */
#define CAN_MARK_EXTENDED_ID                ((uint32_t)0x80000000)


/**
 * @note CAN Communication Settings for CLKSRC = 0; oscillator clock used
 *                                                                                                 \n
 *       @ 8MHz                                                                                    \n
 *       baud        1000000  500000  333333  250000  200000  125000  100000  83333  62500  50000  \n
 *       CANBTR0 =      0x40    0x41    0x      0x      0x      0x47    0x     0x     0x4F   0x    \n
 *       CANBTR1 =      0x23    0x23    0x      0x      0x      0x23    0x     0x     0x23   0x    \n
 *                                                                                                 \n
 *       @ 16 MHz                                                                                  \n
 *       baud        1000000  500000  333333  250000  200000  125000  100000  83333  62500  50000  \n
 *       CANBTR0 =      0x41    0x43    0x45    0x47    0x47    0x4F    0x4F   0x4F   0x5F   0x5F  \n
 *       CANBTR1 =      0x23    0x23    0x23    0x23    0x16    0x23    0x16   0x36   0x23   0x16  \n
 *
 */

/*****************************************************************************
 CAN0 Hardware initialisation
******************************************************************************/
#define CANCTL0_CAN0                        (0x00)          /* clocked in Wait mode, timer disabled, wake-up enabled */
#define CANCTL1_CAN0                        (CANE)          /* MSCAN enabled, clock = OSC_CLK, loop back disabled, listen only disabled, wake-up filter disabled */
#define CANBTR0_CAN0                        (0x40)
#define CANBTR1_CAN0                        (0x23)
#define CANRIER_CAN0                        (RXFIE)         /* receiver buffer full interrupt disable */
#define CANTIER_CAN0                        (0)             /* transmit buffer empty interrupts disabled */

#define CANIDAC_CAN0                        (IDAM0)         /* 4 x 16-bit filters */
#define CANIDAR0_CAN0                       (0)
#define CANIDAR1_CAN0                       (0)
#define CANIDMR0_CAN0                       (0xFFFFFFFF)    /* mask all bits */
#define CANIDMR1_CAN0                       (0xFFFFFFFF)    /* mask all bits */

/*****************************************************************************
 CAN1 Hardware initialisation
******************************************************************************/
#define CANCTL0_CAN1                        (0x00)          /* clocked in Wait mode, timer disabled, wake-up enabled */
#define CANCTL1_CAN1                        (CANE)          /* MSCAN enabled, clock = OSC_CLK, loop back disabled, listen only disabled, wake-up filter disabled */
#define CANBTR0_CAN1                        (0x40)
#define CANBTR1_CAN1                        (0x23)
#define CANRIER_CAN1                        (RXFIE)         /* receiver buffer full interrupt disable */
#define CANTIER_CAN1                        (0)             /* transmit buffer empty interrupts disabled */

#define CANIDAC_CAN1                        (IDAM0)         /* 4 x 16-bit filters */
#define CANIDAR0_CAN1                       (0)
#define CANIDAR1_CAN1                       (0)
#define CANIDMR0_CAN1                       (0xFFFFFFFF)    /* mask all bits */
#define CANIDMR1_CAN1                       (0xFFFFFFFF)    /* mask all bits */

/*****************************************************************************
 CAN2 Hardware initialisation
******************************************************************************/
#define CANCTL0_CAN2                        (0x00)          /* clocked in Wait mode, timer disabled, wake-up enabled */
#define CANCTL1_CAN2                        (CANE)          /* MSCAN enabled, clock = OSC_CLK, loop back disabled, listen only disabled, wake-up filter disabled */
#define CANBTR0_CAN2                        (0x40)
#define CANBTR1_CAN2                        (0x23)
#define CANRIER_CAN2                        (RXFIE)         /* receiver buffer full interrupt disable */
#define CANTIER_CAN2                        (0)             /* transmit buffer empty interrupts disabled */

#define CANIDAC_CAN2                        (IDAM0)         /* 4 x 16-bit filters */
#define CANIDAR0_CAN2                       (0)
#define CANIDAR1_CAN2                       (0)
#define CANIDMR0_CAN2                       (0xFFFFFFFF)    /* mask all bits */
#define CANIDMR1_CAN2                       (0xFFFFFFFF)    /* mask all bits */

/*****************************************************************************
 CAN3 Hardware initialisation
******************************************************************************/
#define CANCTL0_CAN3                        (0x00)          /* clocked in Wait mode, timer disabled, wake-up enabled */
#define CANCTL1_CAN3                        (CANE)          /* MSCAN enabled, clock = OSC_CLK, loop back disabled, listen only disabled, wake-up filter disabled */
#define CANBTR0_CAN3                        (0x40)
#define CANBTR1_CAN3                        (0x23)
#define CANRIER_CAN3                        (RXFIE)         /* receiver buffer full interrupt disable */
#define CANTIER_CAN3                        (0)             /* transmit buffer empty interrupts disabled */

#define CANIDAC_CAN3                        (IDAM0)         /* 4 x 16-bit filters */
#define CANIDAR0_CAN3                       (0)
#define CANIDAR1_CAN3                       (0)
#define CANIDMR0_CAN3                       (0xFFFFFFFF)    /* mask all bits */
#define CANIDMR1_CAN3                       (0xFFFFFFFF)    /* mask all bits */

/*****************************************************************************
 CAN4 Hardware initialisation
******************************************************************************/
#define CANCTL0_CAN4                        (0x00)          /* clocked in Wait mode, timer disabled, wake-up enabled */
#define CANCTL1_CAN4                        (CANE)          /* MSCAN enabled, clock = OSC_CLK, loop back disabled, listen only disabled, wake-up filter disabled */
#define CANBTR0_CAN4                        (0x40)
#define CANBTR1_CAN4                        (0x23)
#define CANRIER_CAN4                        (RXFIE)         /* receiver buffer full interrupt disable */
#define CANTIER_CAN4                        (0)             /* transmit buffer empty interrupts disabled */

#define CANIDAC_CAN4                        (IDAM0)         /* 4 x 16-bit filters */
#define CANIDAR0_CAN4                       (0)
#define CANIDAR1_CAN4                       (0)
#define CANIDMR0_CAN4                       (0xFFFFFFFF)    /* mask all bits */
#define CANIDMR1_CAN4                       (0xFFFFFFFF)    /* mask all bits */


/***************************************************
 *                                                 *
 *  MSCAN Struct                                   *
 *                                                 *
 ***************************************************/
typedef union uCANCTL0
{
    uint8_t  Byte;
    struct
    {
        uint8_t        initrq :1;                           /* initialization mode request */
        uint8_t        slprq  :1;                           /* sleep mode request */
        uint8_t        wupe   :1;                           /* wake-up enable */
        uint8_t        time   :1;                           /* time stamp enable */
        uint8_t        synch  :1;                           /* synchronized status */
        uint8_t        cswai  :1;                           /* msCAN stops in wait mode */
        uint8_t        rxact  :1;                           /* receiver active status */
        uint8_t        rxfrm  :1;                           /* frame recieved flag */
    } Bits;
} tCANCTL0;

#define INITRQ                              (0x01)          /* bit masks */
#define SLPRQ                               (0x02)
#define WUPE                                (0x04)
#define TIME                                (0x08)
#define SYNCH                               (0x10)
#define CSWAI                               (0x20)
#define RXACT                               (0x40)
#define XFRM                                (0x80)

typedef union uCANCTL1
{
    uint8_t  Byte;
    struct
    {
        uint8_t        initak :1;                           /* initialization mode acknowledge */
        uint8_t        slpak  :1;                           /* sleep mode acknowledge*/
        uint8_t        wupm   :1;                           /* wake-up mode low pass filter */
        uint8_t        borm   :1;                           /* bus-off recovery mode */
        uint8_t        listen :1;                           /* listen only mode */
        uint8_t        loopb  :1;                           /* loop back self test mode */
        uint8_t        clksrc :1;                           /* msCAN clock source */
        uint8_t        cane   :1;                           /* msCAN enabled */
    } Bits;
} tCANCTL1;

#define INITAK                              (0x01)          /* bit masks */
#define SLPAK                               (0x02)
#define WUPM                                (0x04)
#define BORM                                (0x08)
#define LISTEN                              (0x10)
#define LOOPB                               (0x20)
#define CLKSRC                              (0x40)
#define CANE                                (0x80)

typedef union uCANBTR0
{
    uint8_t  Byte;
    struct
    {
        uint8_t        brp    :6;                           /* baud rate prescaler */
        uint8_t        sjw    :2;                           /* synchronization jump width */
    } Bits;
} tCANBTR0;

#define BRP0                                (0x01)          /* bit masks */
#define BRP1                                (0x02)
#define BRP2                                (0x04)
#define BRP3                                (0x08)
#define BRP4                                (0x10)
#define BRP5                                (0x20)
#define SJW0                                (0x40)
#define SJW1                                (0x80)

typedef union uCANBTR1
{
    uint8_t  Byte;
    struct
    {
        uint8_t        tseg1  :4;                           /* time segment bits */
        uint8_t        tseg2  :3;
        uint8_t        samp   :1;                           /* sample rate (0:1 per bit;1:3 per bit) */
    } Bits;
} tCANBTR1;

#define TSEG10                              (0x01)          /* bit masks */
#define TSEG11                              (0x02)
#define TSEG12                              (0x04)
#define TSEG13                              (0x08)
#define TSEG20                              (0x10)
#define TSEG21                              (0x20)
#define TSEG22                              (0x40)
#define SAMP                                (0x80)

typedef union uCANRFLG
{
    uint8_t  Byte;
    struct
    {
        uint8_t        rxf    :1;                           /* receive buffer full flag */
        uint8_t        ovrif  :1;                           /* overrun flag */
        uint8_t        tstat  :2;                           /* transmitter status bits */
        uint8_t        rstat  :2;                           /* receiver status bits */
        uint8_t        cscif  :1;                           /* status change interrupt flag  */
        uint8_t        wupif  :1;                           /* wake-up flag */
    } Bits;
} tCANRFLG;

#define RXF                                 (0x01)          /* bit masks */
#define OVRIF                               (0x02)
#define TSTAT0                              (0x04)
#define TSTAT1                              (0x08)
#define RSTAT0                              (0x10)
#define RSTAT1                              (0x20)
#define CSCIF                               (0x40)
#define WUPIF                               (0x80)

typedef union uCANRIER
{
    uint8_t  Byte;
    struct
    {
        uint8_t        rxfie  :1;                           /* receiver full interrupt enable */
        uint8_t        ovrie  :1;                           /* overrun interrupt enable */
        uint8_t        tstate :2;                           /* transmitter status change enable */
        uint8_t        rstate :2;                           /* receiver status change enable */
        uint8_t        cscie  :1;                           /* status change interrupt enable    */
        uint8_t        wupie  :1;                           /* wake-up interrupt enable */
    } Bits;
} tCANRIER;

#define RXFIE                               (0x01)          /* bit masks */
#define OVRIE                               (0x02)
#define TSTATE0                             (0x04)
#define TSTATE1                             (0x08)
#define RSTATE0                             (0x10)
#define RSTATE1                             (0x20)
#define CSCIE                               (0x40)
#define WUPIE                               (0x80)

typedef union uCANTFLG
{
    uint8_t  Byte;
    struct
    {
        uint8_t        txe0   :1;                           /* transmitter buffer 0 empty flag   */
        uint8_t        txe1   :1;                           /* transmitter buffer 1 empty flag   */
        uint8_t        txe2   :1;                           /* transmitter buffer 2 empty flag   */
        uint8_t               :5;                           /* not used */
    } Bits;
} tCANTFLG;

#define TXE0                                (0x01)          /* bit masks */
#define TXE1                                (0x02)
#define TXE2                                (0x04)

typedef union uCANTIER
{
    uint8_t  Byte;
    struct
    {
        uint8_t        txeie0 :1;                           /* transmitter buffer 0 empty interrupt enable */
        uint8_t        txeie1 :1;                           /* transmitter buffer 1 empty interrupt enable */
        uint8_t        txeie2 :1;                           /* transmitter buffer 2 empty interrupt enable */
        uint8_t               :5;                           /* not used */
    } Bits;
} tCANTIER;

#define TXEIE0                              (0x01)          /* bit masks */
#define TXEIE1                              (0x02)
#define TXEIE2                              (0x04)


typedef union uCANTARQ
{
    uint8_t  Byte;
    struct
    {
        uint8_t        abtrq0 :1;                           /* transmitter buffer 0 abort request */
        uint8_t        abtrq1 :1;                           /* transmitter buffer 1 abort request */
        uint8_t        abtrq2 :1;                           /* transmitter buffer 2 abort request */
        uint8_t               :5;                           /* not used */
    } Bits;
} tCANTARQ;

#define ABTRQ0                              (0x01)
#define ABTRQ1                              (0x02)
#define ABTRQ2                              (0x04)

typedef union uCANTAAK
{
    uint8_t  Byte;
    struct
    {
        uint8_t        abtak0 :1;                           /* transmitter buffer 0 abort acknowledge    */
        uint8_t        abtak1 :1;                           /* transmitter buffer 1 abort acknowledge    */
        uint8_t        abtak2 :1;                           /* transmitter buffer 2 abort acknowledge    */
        uint8_t               :5;                           /* not used */
    } Bits;
} tCANTAAK;

#define ABTAK0                              (0x01)
#define ABTAK1                              (0x02)
#define ABTAK2                              (0x04)

typedef union uCANTBSEL
{
    uint8_t  Byte;
    struct
    {
        uint8_t        tx0    :1;                            /* transmitter buffer 0 select   */
        uint8_t        tx1    :1;                            /* transmitter buffer 1 select   */
        uint8_t        tx2    :1;                            /* transmitter buffer 2 select   */
        uint8_t               :5;                            /* not used */
    } Bits;
} tCANTBSEL;

#define TX0                                 (0x01)
#define TX1                                 (0x02)
#define TX2                                 (0x04)

typedef union uCANIDAC
{
    uint8_t  Byte;
    struct
    {
        uint8_t        idhit  :3;                           /* identifier acceptance hit indicator */
        uint8_t               :1;                           /* not used */
        uint8_t        idam0  :1;                           /* identifier acceptance mode */
        uint8_t        idam1  :1;                           /* identifier acceptance mode */
        uint8_t               :2;                           /* not used */
    } Bits;
} tCANIDAC;

#define IDHIT0                              (0x01)          /* bit masks */
#define IDHIT1                              (0x02)
#define IDHIT2                              (0x04)
#define IDAM0                               (0x10)
#define IDAM1                               (0x20)

typedef union uCANMISC
{
    uint8_t  Byte;
    struct
    {
        uint8_t        bohold :1;                           /* bus-off hold bit */
        uint8_t               :7;                           /* not used */
    } Bits;
} tCANMISC;

#define BOHOLD                              (0x01)          /* bit mask */

typedef union uCANDLR
{
    uint8_t            Byte;
    struct
    {
        uint8_t        dlc    :4;                           /* data length code */
        uint8_t        rsv    :4;                           /* not used */
    } Bits;
} tCANDLR;

typedef union uCANID
{
    uint32_t           l;
    uint16_t           w[2];
    uint8_t            b[4];
} tCANID;

typedef struct
{
    tCANID             canidar;
    tCANID             canidmr;
} tCID;

typedef union udsr
{
    uint32_t           l[2];
    uint16_t           w[4];
    uint8_t            b[8];
} tdsr;

typedef struct
{
    tCANID             id;                                  /* message id */
    tdsr               dsr;                                 /* 8 x data bytes */
    tCANDLR            dlr;                                 /* data length value */
    uint8_t            rsv;                                 /* not used */
    uint16_t           tsr;                                 /* time stamp register */
} tRXBUF;

typedef struct
{
    tCANID             id;                                  /* message id */
    tdsr               dsr;                                 /* 8 x data bytes */
    tCANDLR            dlr;                                 /* data length value */
    uint8_t            tbpr;                                /* transmit buffer priority */
    uint16_t           tsr;                                 /* time stamp register */
} tTXBUF;

typedef struct
{
    volatile tCANCTL0  canctl0;                             /* control register 0 */
    volatile tCANCTL1  canctl1;                             /* control register 1 */
    volatile tCANBTR0  canbtr0;                             /* bus timing register 0 */
    volatile tCANBTR1  canbtr1;                             /* bus timing register 1 */
    volatile tCANRFLG  canrflg;                             /* receiver flag register */
    volatile tCANRIER  canrier;                             /* receiver interrupt enable register */
    volatile tCANTFLG  cantflg;                             /* transmitter flag register */
    volatile tCANTIER  cantier;                             /* transmitter interrupt enable register */
    volatile tCANTARQ  cantarq;                             /* transmitter message abort control register */
    volatile tCANTAAK  cantaak;                             /* transmitter message abort flag register */
    volatile tCANTBSEL cantbsel;                            /* transmitter buffer select */
    volatile tCANIDAC  canidac;                             /* identifier acceptance control register */
             uint8_t   rsv0c;                               /* reserved */
    volatile tCANMISC  canmisc;                             /* miscellaneous bits register */
    volatile tREG08    canrxerr;                            /* receive error counter */
    volatile tREG08    cantxerr;                            /* transmit error counter */
    volatile tCID      canid[2];                            /* ID UNION - identifier acceptance/mask registers */
    volatile tRXBUF    rxbuf;                               /* BUFFER STRUCTURE - foreground receive buffer */
    volatile tTXBUF    txbuf;                               /* BUFFER STRUCTURE - foreground transmit buffer */
} tMSCAN;

typedef enum
{
     UNCONNECTED
    ,TJA1040                                                /* high speed */
    ,TJA1041                                                /* high speed */
    ,TJA1050                                                /* high speed */
    ,TJA1054                                                /* low speed */
} tCANTransceiver;

typedef struct
{
    uint8_t            * enable_dir;
    uint8_t            * enable;
    uint8_t            enable_mask;
    uint8_t            * stndby_dir;
    uint8_t            * stndby;
    uint8_t            stndby_mask;
    uint8_t            * error_dir;
    uint8_t            * error;
    uint8_t            error_mask;
} tCANPortPins;
/***************************************************
 *                                                 *
 *  end of MSCAN Struct                            *
 *                                                 *
 ***************************************************/

/*****************************************************************************************************
* Declaration of project wide TYPES
*****************************************************************************************************/
typedef uint8_t t_copyfct(const uint8_t length, const uint8_t *dataptr);
typedef uint8_t t_copyallfct(const uint32_t id, const uint8_t length, const uint8_t *dataptr);

typedef const struct
{
    tCANCTL0           canctl0;                             /* control register 0 */
    tCANCTL1           canctl1;                             /* control register 1 */
    tCANBTR0           canbtr0;                             /* bus timing register 0 */
    tCANBTR1           canbtr1;                             /* bus timing register 1 */
    tCANRIER           canrier;                             /* receiver interrupt enable register */
    tCANTIER           cantier;                             /* transmitter interrupt enable register */
    tCANIDAC           canidac;                             /* identifier acceptance control register */
    tCID               canid[2];                            /* ID UNION - identifier acceptance/mask registers */
    tCANTransceiver    transceiver;                         /* external CAN transceiver */
    tCANPortPins       portpins;                            /* port pins of the connected transceiver */
} CAN_Init_Struct;

typedef union uflag
{
    uint32_t           l;
    uint16_t           w[2];
    uint8_t            b[4];
} tflag;


typedef struct
{
    tMSCAN             *pCAN;                               /* pointer to CAN registers */
    uint8_t            type[CAN_MAX_MSGBUFFER];             /* forwarding rules (periodic...) */
    uint32_t           id[CAN_MAX_MSGBUFFER];               /* identifier */
    uint8_t            len[CAN_MAX_MSGBUFFER];              /* data length */
    uint8_t            data[CAN_MAX_MSGBUFFER][8];          /* data */
    uint16_t           timestamp[CAN_MAX_MSGBUFFER];        /* rx: timestamp of received msg; tx: timer counter */
    uint16_t           timer[CAN_MAX_MSGBUFFER];            /* tx: timer reload value for peiodic sending */
    t_copyfct          *precopyfct[CAN_MAX_MSGBUFFER];      /* copy function */
} CoreCANstruct;

/*****************************************************************************************************
* Definition and declaration of project wide VARIABLES
*****************************************************************************************************/
#ifdef USE_CAN0
    tMSCAN  CAN0   @0x0140;                                 /* MSCAN Module 0 */
    extern CoreCANstruct CoreRXCAN0;
    extern CoreCANstruct CoreTXCAN0;
    extern CAN_Init_Struct CAN_Init_CAN0;
#endif
#ifdef USE_CAN1
    tMSCAN  CAN1   @0x0180;                                 /* MSCAN Module 1 */
    extern CoreCANstruct CoreRXCAN1;
    extern CoreCANstruct CoreTXCAN1;
    extern CAN_Init_Struct CAN_Init_CAN1;
#endif
#ifdef USE_CAN2
    tMSCAN  CAN2   @0x01C0;                                 /* MSCAN Module 2 */
    extern CoreCANstruct CoreRXCAN2;
    extern CoreCANstruct CoreTXCAN2;
    extern CAN_Init_Struct CAN_Init_CAN2;
#endif
#ifdef USE_CAN3
    tMSCAN  CAN3   @0x0200;                                 /* MSCAN Module 3 */
    extern CoreCANstruct CoreRXCAN3;
    extern CoreCANstruct CoreTXCAN3;
    extern CAN_Init_Struct CAN_Init_CAN3;
#endif
#ifdef USE_CAN4
    tMSCAN  CAN4   @0x0280;                                 /* MSCAN Module 4 */
    extern CoreCANstruct CoreRXCAN4;
    extern CoreCANstruct CoreTXCAN4;
    extern CAN_Init_Struct CAN_Init_CAN4;
#endif

/*****************************************************************************************************
* Definition and declaration of project wide (CONST-) CONSTANTS
*****************************************************************************************************/

/*****************************************************************************************************
* Declaration of project wide FUNCTIONS
*****************************************************************************************************/
MOD_EXTERN void CAN_Init(void);
MOD_EXTERN void CAN_InitChannel(CoreCANstruct *pCAN_Channel, CAN_Init_Struct *pCAN_Channel_Init);
MOD_EXTERN void CAN_Set_Standby_Mode(CAN_Init_Struct *pCAN_Channel_Init);
MOD_EXTERN bool_t CAN_ReceiveFct(CoreCANstruct *pCAN_Channel);
MOD_EXTERN void CAN_TransmitFct(CoreCANstruct *pCAN_Channel);
MOD_EXTERN bool_t CAN_TransmitMsg(CoreCANstruct *pCAN_Channel, uint32_t dw_id_l, uint8_t dlc, uint8_t *data);
MOD_EXTERN bool_t CAN_ApplicationTransmitMsg(uint8_t CAN_Channel, uint32_t CAN_Id, uint8_t CAN_Datalength, uint8_t * CAN_Data);
MOD_EXTERN void CAN_SetNotifyFunction(t_copyallfct * pFctAddress);

#ifdef USE_CAN0
    MOD_EXTERN INTERRUPT void CAN_Receive0_ISR(void);
    MOD_EXTERN INTERRUPT void CAN_Transmit0_ISR(void);
#endif

#ifdef USE_CAN1
    MOD_EXTERN INTERRUPT void CAN_Receive0_ISR(void);
    MOD_EXTERN INTERRUPT void CAN_Transmit0_ISR(void);
#endif
#ifdef USE_CAN2
    MOD_EXTERN INTERRUPT void CAN_Receive2_ISR(void);
    MOD_EXTERN INTERRUPT void CAN_Transmit2_ISR(void);
#endif
#ifdef USE_CAN3
    MOD_EXTERN INTERRUPT void CAN_Receive3_ISR(void);
    MOD_EXTERN INTERRUPT void CAN_Transmit3_ISR(void);
#endif
#ifdef USE_CAN4
    MOD_EXTERN INTERRUPT void CAN_Receive4_ISR(void);
    MOD_EXTERN INTERRUPT void CAN_Transmit4_ISR(void);
#endif

/****************************************************************************************************/
#undef MOD_EXTERN
#undef MOD_INIT
#endif  /* #ifndef MODULE_DRV_CAN_H */
